'use strict';

const util = require('../../util');

class SignificantToken {
  constructor(content, line, type, innerHTML) {
    this.content = content;
    this.line = line;
    this.type = type;
    this.innerHTML = innerHTML;

    this.error = undefined; ///
  }
  
  getContent() {
    return this.content;
  }
  
  getLine() {
    return this.line;
  }

  getType() {
    return this.type;
  }

  getInnerHTML() {
    return this.innerHTML;
  }

  getHTML() {
    const className = (this.error === true) ?
                        'error' :
                          this.type,
          html = `<span class="${className}">${this.innerHTML}</span>`;

    return html;
  }

  getLength() {
    return this.content.length; ///
  }

  setError(error) {
    this.error = error;
  }

  clone(startPosition, endPosition) { return SignificantToken.clone(this, startPosition, endPosition, SignificantToken) }

  static clone(token, startPosition = 0, endPosition = token.getLength(), Class = SignificantToken) {
    let clonedSignificantToken = null;

    if (startPosition !== endPosition) {
      const line = token.getLine(),
            type = token.getType();

      let content = token.getContent();

      content = content.substring(startPosition, endPosition);

      clonedSignificantToken = Class.fromContentLineAndType(content, line, type);
    }

    return clonedSignificantToken;
  }

  static fromSignificantToken(significantToken, Class = SignificantToken) {
    const content = significantToken.getContent(),
          line = significantToken.getLine(),
          type = significantToken.getType();
    
    significantToken = Class.fromContentLineAndType(content, line, type);

    return significantToken;
  }

  static fromContentLineAndType(content, line, type, Class = SignificantToken) {
    const innerHTML = Class.innerHTMLFromContent(content),
          significantToken = new Class(content, line, type, innerHTML);

    return significantToken;
  }

  static innerHTMLFromContent(content) {
    const sanitisedContent = util.sanitiseContent(content),
          innerHTML = sanitisedContent; ///

    return innerHTML;
  }
}

SignificantToken.types = {
  string: 'string',
  endOfLine: 'endOfLine',
  whitespace: 'whitespace'
};

module.exports = SignificantToken;
